/*
 * Copyright (c) 2020 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package vcl

import "text/template"

const aclTmplSrc = `
import std;

{{- range .ACLs}}
acl {{aclName .Name}} {
	{{- range .Addresses}}
	{{if .Negate}}! {{end}}"{{.Addr}}"{{aclMask .MaskBits}};
	{{- end}}
}
{{- end}}

sub vcl_recv {
	{{- if hasXFF .ACLs}}
        std.collect(req.http.X-Forwarded-For);
        {{- end}}
	{{- range .ACLs}}
	if (
	    {{- range $cond := .Conditions}}
	    {{$cond.Comparand}} {{cmpRelation .Compare .Negate}} "{{.Value}}" &&
	    {{- end}}
	    {{aclCmp .Comparand}} {{if .Whitelist}}!{{end}}~ {{aclName .Name}}
	   ) {
		{{- if .ResultHdr.Header}}
		set {{.ResultHdr.Header}} = "{{.ResultHdr.Failure}}";
		{{- end}}
		{{- if ge .FailStatus 100}}
		return(synth({{.FailStatus}}));
		{{- end}}
	}
	{{- if .ResultHdr.Header}}
	else {
		set {{.ResultHdr.Header}} = "{{.ResultHdr.Success}}";
	}
	{{- end}}
	{{- end}}
}
`

const aclTmplName = "acl"

var aclTmpl = template.Must(template.New(aclTmplName).Funcs(fMap).
	Parse(aclTmplSrc))
